@@ -17,14 +17,12 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <Python.h>
+#define NO_IMPORT_PYGOBJECT //To avoid a multiple definition, nautilus-python-object.c also includes and does the import.
 #include <pygobject.h>
 #include <gmodule.h>
-#include <gtk/gtk.h>
 
 #include "nautilus-python.h"
 #include "nautilus-python-object.h"
@@ -42,6 +40,17 @@ static gboolean nautilus_python_init_python(void);
 static GArray *all_types = NULL;
 
 
+PyTypeObject *_PyNautilusColumn_Type;
+PyTypeObject *_PyNautilusColumnProvider_Type;
+PyTypeObject *_PyNautilusInfoProvider_Type;
+PyTypeObject *_PyNautilusMenu_Type;
+PyTypeObject *_PyNautilusMenuItem_Type;
+PyTypeObject *_PyNautilusMenuProvider_Type;
+PyTypeObject *_PyNautilusPropertiesItem_Type;
+PyTypeObject *_PyNautilusPropertiesModel_Type;
+PyTypeObject *_PyNautilusPropertiesModelProvider_Type;
+PyTypeObject *_PyNautilusOperationHandle_Type;
+
 static inline gboolean 
 np_init_pygobject(void) {
     PyObject *gobject = pygobject_init (PYGOBJECT_MAJOR_VERSION, PYGOBJECT_MINOR_VERSION, PYGOBJECT_MICRO_VERSION);
@@ -85,9 +94,8 @@ nautilus_python_load_file(GTypeModule *type_module,
 
         if (PyObject_IsSubclass(value, (PyObject*)&PyNautilusColumnProvider_Type) ||
 				PyObject_IsSubclass(value, (PyObject*)&PyNautilusInfoProvider_Type) ||
-				PyObject_IsSubclass(value, (PyObject*)&PyNautilusLocationWidgetProvider_Type) ||
 				PyObject_IsSubclass(value, (PyObject*)&PyNautilusMenuProvider_Type) ||
-				PyObject_IsSubclass(value, (PyObject*)&PyNautilusPropertyPageProvider_Type)) {
+				PyObject_IsSubclass(value, (PyObject*)&PyNautilusPropertiesModelProvider_Type)) {
             gtype = nautilus_python_object_get_type(type_module, value);
             g_array_append_val(all_types, gtype);
         }
@@ -111,12 +119,8 @@ nautilus_python_load_dir (GTypeModule *module,
             
     while ((name = g_dir_read_name(dir))) {
         if (g_str_has_suffix(name, ".py")) {
-            char *modulename;
-            int len;
-
-            len = strlen(name) - 3;
-            modulename = g_new0(char, len + 1 );
-            strncpy(modulename, name, len);
+            size_t len = strlen (name) - 3;
+            g_autofree char *modulename = g_strndup (name, len);
 
             if (!initialized) {
                 PyObject *sys_path, *py_path;
@@ -125,17 +129,12 @@ nautilus_python_load_dir (GTypeModule *module,
                 * at all if no extensions are found) */
                 if (!nautilus_python_init_python()) {
                     g_warning("nautilus_python_init_python failed");
-                    g_dir_close(dir);
                     break;
                 }
                 
                 /* sys.path.insert(0, dirname) */
                 sys_path = PySys_GetObject("path");
-#if PY_MAJOR_VERSION >= 3
                 py_path = PyUnicode_FromString(dirname);
-#else
-                py_path = PyString_FromString(dirname);
-#endif
                 PyList_Insert(sys_path, 0, py_path);
                 Py_DECREF(py_path);
             }
@@ -143,6 +142,8 @@ nautilus_python_load_dir (GTypeModule *module,
             nautilus_python_load_file(module, modulename);
         }
     }    
+
+    g_dir_close (dir);
 }
 
 static gboolean
@@ -153,8 +154,8 @@ nautilus_python_init_python (void) {
     if (Py_IsInitialized())
         return TRUE;
 
-    debug("g_module_open " PY_LIB_LOC "/lib" PY_LIB_NAME "." G_MODULE_SUFFIX ".1.0");  
-    libpython = g_module_open (PY_LIB_LOC "/lib" PY_LIB_NAME "." G_MODULE_SUFFIX ".1.0", 0);
+    debug("g_module_open " PYTHON_LIBPATH);
+    libpython = g_module_open (PYTHON_LIBPATH, 0);
     if (!libpython)
         g_warning("g_module_open libpython failed: %s", g_module_error());
 
@@ -166,11 +167,7 @@ nautilus_python_init_python (void) {
     }
     
     debug("PySys_SetArgv");
-#if PY_MAJOR_VERSION >= 3
     wchar_t *argv[] = { L"nautilus", NULL };
-#else
-    char *argv[] = { "nautilus", NULL };
-#endif
     PySys_SetArgv(1, argv);
     if (PyErr_Occurred()) {
         PyErr_Print();
@@ -194,16 +191,13 @@ nautilus_python_init_python (void) {
     /* import nautilus */
     g_setenv("INSIDE_NAUTILUS_PYTHON", "", FALSE);
     debug("import nautilus");
-    PyRun_SimpleString("import gi; gi.require_version('Nautilus', '3.0')");
+    PyRun_SimpleString("import gi; gi.require_version('Nautilus', '4.0')");
     nautilus = PyImport_ImportModule("gi.repository.Nautilus");
     if (!nautilus) {
         PyErr_Print();
         return FALSE;
     }
 
-    _PyGtkWidget_Type = pygobject_lookup_class(GTK_TYPE_WIDGET);
-    g_assert(_PyGtkWidget_Type != NULL);
-
 #define IMPORT(x, y) \
     _PyNautilus##x##_Type = (PyTypeObject *)PyObject_GetAttrString(nautilus, y); \
     if (_PyNautilus##x##_Type == NULL) { \
@@ -214,12 +208,12 @@ nautilus_python_init_python (void) {
     IMPORT(Column, "Column");
     IMPORT(ColumnProvider, "ColumnProvider");
     IMPORT(InfoProvider, "InfoProvider");
-    IMPORT(LocationWidgetProvider, "LocationWidgetProvider");
     IMPORT(Menu, "Menu");
     IMPORT(MenuItem, "MenuItem");
     IMPORT(MenuProvider, "MenuProvider");
-    IMPORT(PropertyPage, "PropertyPage");
-    IMPORT(PropertyPageProvider, "PropertyPageProvider");
+    IMPORT(PropertiesItem, "PropertiesItem");
+    IMPORT(PropertiesModel, "PropertiesModel");
+    IMPORT(PropertiesModelProvider, "PropertiesModelProvider");
     IMPORT(OperationHandle, "OperationHandle");
 
 #undef IMPORT
@@ -230,8 +224,6 @@ nautilus_python_init_python (void) {
 
 static void
 nautilus_python_check_all_directories(GTypeModule *module) {
-    gchar *extensions_dir = NULL;
-
     GList *dirs = NULL;
 
     // Check ~/.local/share first
@@ -241,7 +233,7 @@ nautilus_python_check_all_directories(GTypeModule *module) {
     // If nautilus is built in a non-standard prefix
     // Check nautilus prefix's DATADIR
     gchar *prefix_extension_dir = DATADIR "/nautilus-python/extensions";
-    dirs = g_list_append(dirs, prefix_extension_dir);
+    dirs = g_list_append(dirs, g_strdup (prefix_extension_dir));
 
     // Check all system data dirs 
     const gchar *const *temp = g_get_system_data_dirs();
@@ -250,6 +242,8 @@ nautilus_python_check_all_directories(GTypeModule *module) {
             "nautilus-python", "extensions", NULL);
         if (g_strcmp0(dir, prefix_extension_dir) != 0) {
             dirs = g_list_append(dirs, dir);
+        } else {
+            g_free (dir);
         }
 
         temp++;
@@ -262,12 +256,11 @@ nautilus_python_check_all_directories(GTypeModule *module) {
         dirs = dirs->next;
     }
 
-    g_list_free(dirs);
+    g_list_free_full (dirs, g_free);
 }
 
 void
 nautilus_module_initialize(GTypeModule *module) {
-    gchar *user_extensions_dir;
     const gchar *env_string;
 
     env_string = g_getenv("NAUTILUS_PYTHON_DEBUG");
